<?php

namespace FiloBlu\AdminSecurity\Controller\Adminhtml\Auth;

use FiloBlu\AdminSecurity\Model\Authentication;
use FiloBlu\AdminSecurity\Model\Ip;
use FiloBlu\AdminSecurity\Model\TrustedManager;
use Magento\Backend\App\Action\Context;
use Magento\Backend\Model\Auth\Session;
use Magento\Security\Model\AdminSessionsManager;
use Magento\User\Model\UserFactory;

class Resetpassword extends \FiloBlu\AdminSecurity\Controller\Adminhtml\Auth
{
    /**
     * @var Authentication
     */
    protected $authModel;

    /**
     * @var Session
     */
    protected $authStorage;

    /**
     * @var AdminSessionsManager
     */
    protected $authManager;

    /**
     * @var TrustedManager
     */
    protected $trustedManager;

    /**
     * @param Context $context
     * @param UserFactory $userFactory
     * @param Authentication $authModel
     * @param Ip $ipModel
     * @param Session $authStorage
     * @param AdminSessionsManager $authManager
     */
    public function __construct(
        Context              $context,
        UserFactory          $userFactory,
        Authentication       $authModel,
        Ip                   $ipModel,
        Session              $authStorage,
        AdminSessionsManager $authManager,
        TrustedManager       $trustedManager
    )
    {
        parent::__construct(
            $context,
            $userFactory,
            $authModel,
            $ipModel
        );
        $this->authModel = $authModel;
        $this->authStorage = $authStorage;
        $this->authManager = $authManager;
        $this->trustedManager = $trustedManager;
    }

    /**
     * Send admin one time password action
     *
     * @return void
     */
    public function execute()
    {
        $user = $this->authStorage->getTfaUser();
        $password = (string)$this->getRequest()->getPost('password');

        if (!$this->ipModel->isIpInWhiteList()) {
            // show verification form
            if ($this->authStorage->getNeedVerification()) {
                $this->authStorage->setNeedVerification(false);
                if (is_null($this->authStorage->getUserSecData())) {
                    $this->authStorage->setUserSecData($this->authModel->createAccountSecret());
                }

                $this->messageManager->getMessages(true);
                $this->messageManager->addSuccess(
                    __('The email with verification code has been succesfully sent to your account address.')
                );

                $this->_view->loadLayout();
                $this->_view->renderLayout();
                return;
            } else {
                if (!is_null($this->authStorage->getUserSecData())) {
                    if ($this->authModel->userAllowed($this->authStorage->getUserSecData(), $password)) {
                        $this->authStorage
                            ->setTfaUser(null)
                            ->setUser($user)
                            ->processLogin();

                        if ($this->authStorage->isLoggedIn()) {
                            $this->trustedManager->handleTrustDeviceRequest($this->getRequest(), $user);
                            $this->authManager->processLogin();
                            $this->authStorage->setUserSecData(null);
                            return $this->_homeRedirect();
                        }
                    }
                }
            }
        }

        $this->messageManager->addErrorMessage(
            __('You did not sign in correctly or access restricted.')
        );
        return $this->_homeRedirect();
    }

    protected function _homeRedirect($url = null)
    {
        if (is_null($url)) {
            $url = $this->_objectManager->get(\Magento\Backend\Helper\Data::class)->getHomePageUrl();
        }
        return $this->getResponse()->setRedirect($url);
    }
}
