<?php

namespace FiloBlu\AdminSecurity\Observer;

use FiloBlu\AdminSecurity\Helper\Data;
use FiloBlu\AdminSecurity\Model\Authentication;
use FiloBlu\AdminSecurity\Model\Ip;
use Magento\Backend\App\ConfigInterface;
use Magento\Backend\Model\Auth\Session;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Exception\Plugin\AuthenticationException;
use Magento\Framework\Mail\Template\TransportBuilder;
use Magento\Store\Model\Store;
use Magento\User\Model\UserFactory;

class LoginBackendObserver implements ObserverInterface
{
    /**
     * @var TransportBuilder
     */
    protected $transportBuilder;

    /**
     * @var ConfigInterface
     */
    protected $config;

    /**
     * @var RequestInterface
     */
    protected $request;

    /**
     * @var Authentication
     */
    protected $authModel;

    /**
     * @var Ip
     */
    protected $ipModel;

    /**
     * @var Session
     */
    protected $authStorage;

    /**
     * @var UserFactory
     */
    protected $userFactory;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @param RequestInterface $request
     * @param Authentication $authModel
     * @param Ip $ipModel
     * @param UserFactory $userFactory
     * @param Session $authStorage
     * @param TransportBuilder $transportBuilder
     * @param ConfigInterface $config
     * @param Data $helper
     */
    public function __construct(
        RequestInterface $request,
        Authentication $authModel,
        Ip $ipModel,
        UserFactory $userFactory,
        Session $authStorage,
        TransportBuilder $transportBuilder,
        ConfigInterface $config,
        Data $helper
    ) {
        $this->request = $request;
        $this->authModel = $authModel;
        $this->ipModel = $ipModel;
        $this->userFactory = $userFactory;
        $this->authStorage = $authStorage;
        $this->transportBuilder = $transportBuilder;
        $this->config = $config;
        $this->helper = $helper;
    }

    /**
     * Check Login Backend
     *
     * @param Observer $observer
     * @throws AuthenticationException
     */
    public function execute(Observer $observer)
    {
        if (!$this->authStorage->getAllowBackendAccountLoginCheck()) {
            return;
        }
        $this->authStorage->setAllowBackendAccountLoginCheck(false);

        $user         = $this->userFactory->create()->loadByUsername($observer->getEvent()->getUsername());
        $otpPassword  = $this->request->getPost('otp_password');
        $loginSuccess = $observer->getEvent()->getResult();
        
        if ($loginSuccess && $user->getId()) {
            
            if(!is_null($this->authStorage->getUserSecData())){
                if (!$this->authModel->userAllowed($this->authStorage->getUserSecData(), $otpPassword)) {
                    // enable second login step
                    if (empty($otpPassword) && $this->helper->adminSecurityIsEnabled()) {
                        $this->_sendAdminOtpEmail($user, $this->authModel->createHotp($this->authStorage->getUserSecData()));
                        // prepare verification form
                        $this->authStorage->setNeedVerification(true);
                        $this->authStorage->setTfaUser($user);
                        $this->authStorage->setUser(null);
                    }

                    throw new AuthenticationException(__('You did not sign in correctly or access restricted.'));
                }
            }
        }
    }
    
    /**
     * Send admin one time password action
     *
     * @param mixed $user
     * @param string $newPassword
     *
     * @return void
     */
    protected function _sendAdminOtpEmail($user, $newPassword)
    {
        $transport = $this->transportBuilder->setTemplateIdentifier(
            \FiloBlu\AdminSecurity\Helper\Data::OTP_EMAIL_TEMPLATE
        )
            ->setTemplateModel(\Magento\Email\Model\BackendTemplate::class)
            ->setTemplateOptions([
                'area'  => \Magento\Framework\App\Area::AREA_ADMINHTML,
                'store' => Store::DEFAULT_STORE_ID
            ])
            ->setTemplateVars([
                'user'     => $user,
                'password' => $newPassword
            ])
            ->setFrom($this->config->getValue(\Magento\User\Model\User::XML_PATH_FORGOT_EMAIL_IDENTITY))
            ->addTo($user->getEmail(), $user->getName())
            ->getTransport();

        $transport->sendMessage();
    }
}
